@@ -1,6 +1,212 @@
+#' Permutation test for general dependent truncation
+#'
+#' Perform permutation test based on conditional or unconditional approach.
+#'
+#' @param trun is the left truncation time.
+#' @param obs is the observed failure time.
+#' @param permSize is the number of permutations.
+#' @param cens is the status indicator; 0 = censored, 1 = event.
+#' @param sampling a character string specifying the sampling method used in permutation.
+#' The following are permitted:
+#' \describe{
+#' \item{\code{conditional}}{conditional permutation;}
+#' \item{\code{uconditional}}{unconditional permutation;}
+#' \item{\code{is.conditional}}{importance sampling version of conditional permutation;}
+#' \item{\code{is.uconditional}}{importance sampling version of unconditional permutation;}
+#' }
+#' @param kendallOnly,minp1Only,minp2Only optional values indicating which test statistics to be used.
+#' If all leave as \code{FALSE}, \code{permDep} will use all three test statistics in each permutation.
+#' @param nc is the number of cores used in permutation.
+#' When \code{nc > 1}, permutation is carried out with parallel computing.
+#' @param seed an optional vector containing random seeds to be used to generate permutation samples.
+#' Random seeds will be used when left unspecified.
+#'
+#' @return A list containing output with the following components:
+#' \describe{
+#' \item{obsKen}{the observed p-value using Kendall's tau test statistic.}
+#' \item{obsP1}{the observed p-value using minp1 test statistic.}
+#' \item{obsP2}{the observed p-value using minp2 test statistic.}
+#' \item{obsTest1}{the observed minp1 test statistic.}
+#' \item{obsTest2}{the observed minp2 test statistic.}
+#' \item{permKen}{Kendall's tau test statistics from permutation samples.}
+#' \item{permP1}{minp1 test statistics from permutation samples.}
+#' \item{permP2}{minp2 test statistics from permutation samples.}
+#' }
+#'
+#' @references Chiou, S.H., Qian, J., and Betensky, R.A. (2017).
+#' Permutation Test for General Dependent Truncation. \emph{Tech-report}
+#'
+#' @importFrom BB spg
+#' @importFrom survival Surv survfit basehaz coxph
+#' @importFrom stats model.matrix pchisq pnorm rexp runif var
+#' @importFrom parallel detectCores parLapply makeCluster clusterExport stopCluster
+#'
+#' @export
+#' @keywords permDep
+#' 
+#' @examples
+#' simDat <- function(n) {
+#'   k <- s <- 1
+#'   tt <- xx <- yy <- cc <- delta <- rep(-1, n)
+#'   while(k <= n){
+#'     tt[k] <- runif(1, 0, 3.5)
+#'     xx[k] <- 1.95 + 0.65 * (tt[k] - 1.25)^2 + rnorm(1, sd = 0.1)
+#'     cc[k] <- runif(1, 0, 10)
+#'     delta[k] <- (xx[k] <= cc[k])
+#'     yy[k] <- pmin(xx[k], cc[k])
+#'     s <- s + 1
+#'     if(tt[k] <= yy[k]) k = k+1
+#'   }
+#'  data.frame(list(trun = tt, obs = yy, delta = delta))
+#' }
+#'
+#' set.seed(123)
+#' dat <- simDat(50)
+#' B <- 20
+#'
+#' ## Perform conditional permutation with Kendall's tau, minp1 and minp2
+#' set.seed(123)
+#' system.time(fit <- with(dat, permDep(trun, obs, B, delta, nc = 1)))
+#' fit
+permDep <- function(trun, obs, permSize, cens,
+                    sampling = c("conditional", "unconditional", "is.conditional", "is.unconditional"),
+                    kendallOnly = FALSE, minp1Only = FALSE, minp2Only = FALSE,
+                    nc = ceiling(detectCores() / 2), seed = NULL) {
+    sampling <- match.arg(sampling)
+    n <- length(obs)
+    ## if (!(sampling %in% c("cond", "ucond", "ucond1", "ucond2",
+    ##                       "ucond3", "ucond4", "iscond", "isucond")))
+    obsKen <- obsKenp <- obs1 <- obs2 <- obsP1 <- obsP2 <- obsTest1 <- obsTest2 <- NULL
+    if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
+        tmp <- getKendall(trun, obs, cens)
+        obsKen <- tmp[1]
+        obsKenp <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
+        obs1 <- getMinP(trun, obs, cens)
+        obsP1 <- obs1$minP
+        obsTest1 <- obs1$maxTest
+        obs2 <- getMinP(trun, obs, cens, minp1 = FALSE)
+        obsP2 <- obs2$minP
+        obsTest2 <- obs2$maxTest
+    }
+    if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
+        if (kendallOnly) {
+            tmp <- getKendall(trun, obs, cens)
+            obsKen <- tmp[1]
+            obsKenp <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
+        }
+        if (minp1Only) {        
+            obs1 <- getMinP(trun, obs, cens)
+            obsP1 <- obs1$minP
+            obsTest1 <- obs1$maxTest
+        }
+        if (minp2Only) {
+            obs2 <- getMinP(trun, obs, cens, minp1 = FALSE)
+            obsP2 <- obs2$minP
+            obsTest2 <- obs2$maxTest
+        }
+    }
+    MinP1Time <- MinP2Time <- matrix(NA, permSize, length(trun))
+    if (is.null(seed) || length(seed) != permSize) seed <- sample(.Machine$integer.max, size = permSize)
+    if (nc == 1) {
+        permKen <- permKenp <- permP1 <- permP2 <- permTest1 <- permTest2<- rep(NA, permSize)
+        for(i in 1:permSize){
+            perm.data <- getPerm(trun, obs, cens, sampling, seed[i])    
+            if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
+                if (kendallOnly) {
+                    tmp <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)
+                    permKen[i] <- tmp[1]
+                    permKenp[i] <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
+                }
+                if (minp1Only) {
+                    p1fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
+                    permTest1[i] <- p1fit$maxTest
+                }
+                if (minp2Only) {
+                    p2fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)
+                    permTest2[i] <- p2fit$maxTest
+                }
+            }
+            if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
+                kendallOnly <- minp1Only <- minp2Only <- TRUE
+                tmp <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)
+                permKen[i] <- tmp[1]
+                permKenp[i] <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
+                p1fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
+                                 ## obsTest = ifelse(surv, NA, obsTest1))
+                p2fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)
+                                 ## obsTest = ifelse(surv, NA, obsTest2), minp1 = FALSE)  
+                permTest1[i] <- p1fit$maxTest
+                permTest2[i] <- p2fit$maxTest
+                ## MinP1Time[i,] <- p1fit$MinPTime
+                ## MinP2Time[i,] <- p2fit$MinPTime
+            }
+        }
+    }    
+    if (nc > 1) {
+        permKen <- permP1 <- permP2 <- permTest1 <- permTest2<- rep(NULL, permSize)
+        cl <- makeCluster(nc)
+        clusterExport(cl = cl, varlist=c("trun", "obs", "cens", "sampling", "seed", "obsTest1", "obsTest2"),
+                      envir = environment())
+        clusterExport(cl = cl, varlist=c("getPerm", "getKendall", "getMinP"), envir = environment())
+        out <- unlist(parLapply(cl, 1:permSize, function(x) {
+            perm.data <- getPerm(trun, obs, cens, sampling, seed[x])
+            if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
+                tmpK <- tmpP1 <- tmpP2 <- NULL
+                if (kendallOnly)
+                    tmpK <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)[1]
+                if (minp1Only) 
+                    tmpP1 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
+                if (minp2Only)                                     
+                    tmpP2 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)
+                return(c(tmpK, tmpP1$maxTest, tmpP2$maxTest))
+            }
+            if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
+                kendallOnly <- minp1Only <- minp2Only <- TRUE
+                tmpK <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)[1]
+                tmpP1 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
+                ## obsTest = ifelse(surv, 1e5, obsTest1))
+                tmpP2 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)  
+                ## obsTest = ifelse(surv, 1e5, obsTest2), minp1 = FALSE)  
+                return(c(tmpK, tmpP1$maxTest, tmpP2$maxTest))
+            }}))
+        stopCluster(cl)
+        out <- matrix(out, nrow = ifelse(sum(kendallOnly, minp1Only, minp2Only) == 0,
+                                         3, sum(kendallOnly, minp1Only, minp2Only)))
+        tmp <- matrix(NA, 3, permSize)
+        if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
+            tmp <- out
+            kendallOnly <- minp1Only <- minp2Only <- TRUE
+        } else {
+            tmp[(1:3)[c(kendallOnly, minp1Only, minp2Only)],] <- out
+        }
+        permKen <- tmp[1,]
+        permTest1 <- tmp[2,]
+        permTest2 <- tmp[3,]
+    }
+    if (!is.null(permTest1) && all(!is.na(permTest1))) {
+        ## perm.data.P1 <- getPerm(trun, obs, cens, sampling, seed[which.min(permTest1)])
+        permP1 <- 1 - pchisq(permTest1, df = 1)
+    }
+    if (!is.null(permTest2) && all(!is.na(permTest2))) {
+        ## perm.data.P2 <- getPerm(trun, obs, cens, sampling, seed[which.min(permTest2)])
+        permP2 <- 1 - pchisq(permTest2, df = 1)
+    }
+    out <- list(obsKen = obsKen, obsKenp = obsKenp, obsP1 = obsP1, obsP2 = obsP2,
+                obsTest1 = obsTest1, obsTest2 = obsTest2,
+                permKen = permKen, permP1 = permP1, permP2 = permP2,
+                permTest1 = permTest1, permTest2 = permTest2,
+                p.valueKen = ifelse(kendallOnly, (sum(abs(obsKen) < abs(permKen)) + 1)/(length(permKen) + 1), NA),
+                p.valueMinp1 = ifelse(minp1Only, (sum(obsP1 > permP1) + 1) / (length(permKen) + 1), NA),
+                p.valueMinp2 = ifelse(minp2Only, (sum(obsP2 > permP2) + 1) / (length(permKen) + 1), NA),
+                kendallOnly = kendallOnly, minp1Only = minp1Only, minp2Only = minp2Only,
+                sampling = sampling, permSize = permSize, random.seed = seed)
+    class(out) <- "permDep"
+    return(out)
+}
+
 mysample <- function(x, n) {
     res <- vector("integer", length(x))
-    .C("mysample", as.integer(x), as.integer(n),
+    .C("mysampleC", as.integer(x), as.integer(n),
        as.integer(length(x)), out = as.integer(res), PACKAGE = "permDep")$out
 }
 
@@ -23,8 +229,6 @@ getKendallWgt <- function(trun, obs, cens = NULL, scX = NULL, scT = NULL) {
 }
 
 getScore <- function(x, y) {
-    ## x is a covariate matrix
-    ## y is a survival object with left truncation, right censoring
     n <- nrow(y)
     nvar <- ncol(x)
     start <- y[, 1]
@@ -50,9 +254,6 @@ getScore <- function(x, y) {
 ## control <- list(eps = 1e-09, toler.chol = 1.818989e-12, iter.max = 20, toler.inf = 3.162278e-05, outer.max = 10)
 
 getMinP <- function(trun, obs, cens, obsTest = NA, minp1 = TRUE, eps = NULL) {
-    ## trun (n by 1) is the left truncation times
-    ## obs (n by 1) is the observed failure time
-    ## cens (n by 1) is the censoring indicator: 1-failure, 0-censored
     E <- min(10, round(0.2 * sum(cens)), round(0.2 * length(obs)))
     n <- length(obs)
     data0 <- data.frame(cbind(trun, obs, cens))[order(trun),]
@@ -105,7 +306,7 @@ getPerm <- function(trun, obs, cens, sampling, seed = NULL) {
     tt <- trun
     if (!is.null(seed))
         set.seed(seed)
-    if (sampling == "cond") {
+    if (sampling == "conditional") {
         obs.sort <- sort(obs)
         cens.sort <- cens[order(obs)]
         trun.sort <- sort(trun)
@@ -119,49 +320,45 @@ getPerm <- function(trun, obs, cens, sampling, seed = NULL) {
         perm.trun <- trun.sort[perm.trun.index]
         out <- list(trun = perm.trun, obs = obs.sort, cens = cens.sort)
     }
-    if (sampling == "ucond") {
+    if (sampling == "unconditional") {
         perm.trun.index <- sample(1:n, n) 
         perm.trun.0 <- trun[perm.trun.index]
         out <- list(trun = perm.trun.0, obs = obs, cens = cens)
     }
-    if (sampling == "ucond1") {
+    if (sampling == "unconditional1") {
         perm.trun.index <- sample(1:n, n) 
         perm.trun.0 <- trun[perm.trun.index]
         out <- list(trun = perm.trun.0, obs = obs, cens = cens)
         out <- subset(data.frame(out), trun < obs & tt < trun)
     }
-    if (sampling == "ucond2") {
+    if (sampling == "unconditional2") {
         perm.trun.index <- sample(1:n, n) 
         perm.trun.0 <- trun[perm.trun.index]
         out <- list(trun = perm.trun.0, obs = obs, cens = cens)
         out$trun <- pmax(tt, out$trun)
     }
-    if (sampling == "ucond3") {
+    if (sampling == "unconditional3") {
         perm.trun.index <- sample(1:n, n, replace = TRUE)
         perm.trun.0 <- trun[perm.trun.index]
         out <- list(trun = perm.trun.0, obs = obs, cens = cens)        
     }
-    if (sampling == "ucond4") {
+    if (sampling == "unconditional4") {
         perm.trun.index <- unlist(lapply(sapply(1:n, function(x) which(trun[x] < obs)), function(y) ifelse(length(y) == 1, y, sample(y, 1))))
         perm.trun.0 <- trun[perm.trun.index]
         out <- list(trun = perm.trun.0, obs = obs, cens = cens)        
     }
-    if (sampling == "iscond") {
+    if (sampling == "is.conditional") {
         A <- 1 * sapply(obs, function(x) x > trun)
         trun <- trun[order(rowSums(A))]
         for (maxit in 1:50) {
             A2 <- A[order(rowSums(A)),]
             sp <- NULL
             for (i in 1:n) {
-                ## prob <- exp((n - colSums(A)) / (n - 1))
-                ## if (max(0, prob[sz], na.rm = TRUE) > 0 & sum(A2[i,] > 0) > 0)
                 if (sum(A2[i,] > 0) > 0) {
                     sz <- which(A2[i,] > 0)
                     prob <- exp((n - i + 1 - colSums(A2)) / (n - i))
-                    if (length(sz) == 1) 
-                        sp <- c(sp, sz)
-                    else
-                        sp <- c(sp, sample(sz, 1, TRUE, prob[sz]))
+                    if (length(sz) == 1) sp <- c(sp, sz)
+                    else sp <- c(sp, sample(sz, 1, TRUE, prob[sz]))
                     } else break
                 A2[,sp] <- 0
                 A2[i,] <- 0
@@ -174,18 +371,15 @@ getPerm <- function(trun, obs, cens, sampling, seed = NULL) {
             stop("Permutation sample not found", call. = FALSE)
         out <- list(trun = trun, obs = obs[sp], cens = cens[sp])
     }
-    if (sampling == "isucond") {
+    if (sampling == "is.unconditional") {
         A <- 1 * sapply(obs, function(x) x > trun)
         trun <- trun[order(rowSums(A))]
         A <- A[order(rowSums(A)),]
         sp <- NULL
         for (i in 1:n) {
-            ## prob <- exp((n - colSums(A)) / (n - 1))
             prob <- ifelse(colSums(A) != 0, exp((n - i + 1 - colSums(A)) / (n - i)), 0)
-            if (i == n)
-                sp <- c(sp, which(!(1:n %in% sp)))
-            else
-                sp <- c(sp, sample(1:n, 1, TRUE, prob))
+            if (i == n) sp <- c(sp, which(!(1:n %in% sp)))
+            else sp <- c(sp, sample(1:n, 1, TRUE, prob))
             A[,sp] <- 0
             A[i,] <- 0            
         }
@@ -194,169 +388,3 @@ getPerm <- function(trun, obs, cens, sampling, seed = NULL) {
     out <- subset(data.frame(out), trun < obs )
     out[order(out[,1]),]
 }
-
-permDep <- function(trun, obs, permSize, cens, sampling = "cond", 
-                     kendallOnly = FALSE, minp1Only = FALSE, minp2Only = FALSE,
-                     nc = ceiling(detectCores() / 2), seed = NULL) {
-    ## obs: the failure time, which is subject to left truncation by trun and right censroing
-    ## observable:  obs >= trun
-    ## trun: the truncation time
-    ## permSize:  number of random samples from permutation distribution
-    n <- length(obs)
-    ## error check
-    if (!(sampling %in% c("cond", "ucond", "ucond1", "ucond2", "ucond3", "ucond4", "iscond", "isucond")))
-        stop("Invalid sampling method", call. = FALSE)
-    obsKen <- obsKenp <- obs1 <- obs2 <- obsP1 <- obsP2 <- obsTest1 <- obsTest2 <- NULL
-    if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
-        tmp <- getKendall(trun, obs, cens)
-        obsKen <- tmp[1]
-        obsKenp <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
-        obs1 <- getMinP(trun, obs, cens)
-        obsP1 <- obs1$minP
-        obsTest1 <- obs1$maxTest
-        obs2 <- getMinP(trun, obs, cens, minp1 = FALSE)
-        obsP2 <- obs2$minP
-        obsTest2 <- obs2$maxTest
-    }
-    if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
-        if (kendallOnly) {
-            tmp <- getKendall(trun, obs, cens)
-            obsKen <- tmp[1]
-            obsKenp <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
-        }
-        if (minp1Only) {        
-            obs1 <- getMinP(trun, obs, cens)
-            obsP1 <- obs1$minP
-            obsTest1 <- obs1$maxTest
-        }
-        if (minp2Only) {
-            obs2 <- getMinP(trun, obs, cens, minp1 = FALSE)
-            obsP2 <- obs2$minP
-            obsTest2 <- obs2$maxTest
-        }
-    }
-    MinP1Time <- MinP2Time <- matrix(NA, permSize, length(trun))
-    if (is.null(seed) || length(seed) != permSize) seed <- sample(.Machine$integer.max, size = permSize)
-    if (nc == 1) {
-        permKen <- permKenp <- permP1 <- permP2 <- permTest1 <- permTest2<- rep(NA, permSize)
-        for(i in 1:permSize){
-            perm.data <- getPerm(trun, obs, cens, sampling, seed[i])    
-            if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
-                if (kendallOnly) {
-                    tmp <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)
-                    permKen[i] <- tmp[1]
-                    permKenp[i] <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
-                }
-                if (minp1Only) {
-                    p1fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                    permTest1[i] <- p1fit$maxTest
-                }
-                if (minp2Only) {
-                    p2fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                    permTest2[i] <- p2fit$maxTest
-                }
-            }
-            if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
-                kendallOnly <- minp1Only <- minp2Only <- TRUE
-                tmp <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)
-                permKen[i] <- tmp[1]
-                permKenp[i] <- 2 - 2 * pnorm(abs(tmp[1] / sqrt(tmp[2])))
-                p1fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                                 ## obsTest = ifelse(surv, NA, obsTest1))
-                p2fit <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                                 ## obsTest = ifelse(surv, NA, obsTest2), minp1 = FALSE)  
-                permTest1[i] <- p1fit$maxTest
-                permTest2[i] <- p2fit$maxTest
-                ## MinP1Time[i,] <- p1fit$MinPTime
-                ## MinP2Time[i,] <- p2fit$MinPTime
-            }
-        }
-    }    
-    if (nc > 1) {
-        permKen <- permP1 <- permP2 <- permTest1 <- permTest2<- rep(NULL, permSize)
-        cl <- makeCluster(nc)
-        clusterExport(cl = cl, varlist=c("trun", "obs", "cens", "sampling", "seed", "obsTest1", "obsTest2"),
-                      envir = environment())
-        clusterExport(cl = cl, varlist=c("getPerm", "getKendall", "getMinP"), envir = environment())
-        out <- unlist(parLapply(cl, 1:permSize, function(x) {
-            perm.data <- getPerm(trun, obs, cens, sampling, seed[x])
-            if (sum(kendallOnly, minp1Only, minp2Only) > 0) {
-                tmpK <- tmpP1 <- tmpP2 <- NULL
-                if (kendallOnly)
-                    tmpK <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)[1]
-                if (minp1Only) 
-                    tmpP1 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                if (minp2Only)                                     
-                    tmpP2 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)
-                return(c(tmpK, tmpP1$maxTest, tmpP2$maxTest))
-            }
-            if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
-                kendallOnly <- minp1Only <- minp2Only <- TRUE
-                tmpK <- getKendall(perm.data$trun, perm.data$obs, perm.data$cens)[1]
-                tmpP1 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens)
-                ## obsTest = ifelse(surv, 1e5, obsTest1))
-                tmpP2 <- getMinP(perm.data$trun, perm.data$obs, perm.data$cens, minp1 = FALSE)  
-                ## obsTest = ifelse(surv, 1e5, obsTest2), minp1 = FALSE)  
-                return(c(tmpK, tmpP1$maxTest, tmpP2$maxTest))
-            }}))
-        stopCluster(cl)
-        out <- matrix(out, nrow = ifelse(sum(kendallOnly, minp1Only, minp2Only) == 0,
-                                         3, sum(kendallOnly, minp1Only, minp2Only)))
-        tmp <- matrix(NA, 3, permSize)
-        if (sum(kendallOnly, minp1Only, minp2Only) == 0) {
-            tmp <- out
-            kendallOnly <- minp1Only <- minp2Only <- TRUE
-        } else {
-            tmp[(1:3)[c(kendallOnly, minp1Only, minp2Only)],] <- out
-        }
-        permKen <- tmp[1,]
-        permTest1 <- tmp[2,]
-        permTest2 <- tmp[3,]
-    }
-    perm.data.Ken <- perm.data.P1 <- perm.data.P2 <- NULL
-    if (!is.null(permKen) && all(!is.na(permKen))) 
-        perm.data.Ken <- getPerm(trun, obs, cens, sampling, seed[which.min(abs(permKen))])
-    if (!is.null(permTest1) && all(!is.na(permTest1))) {
-        perm.data.P1 <- getPerm(trun, obs, cens, sampling, seed[which.min(permTest1)])
-        permP1 <- 1 - pchisq(permTest1, df = 1)
-    }
-    if (!is.null(permTest2) && all(!is.na(permTest2))) {
-        perm.data.P2 <- getPerm(trun, obs, cens, sampling, seed[which.min(permTest2)])
-        permP2 <- 1 - pchisq(permTest2, df = 1)
-    }
-    out <- list(obsKen = obsKen, obsKenp = obsKenp, obsP1 = obsP1, obsP2 = obsP2,
-                obsTest1 = obsTest1, obsTest2 = obsTest2,
-                permKen = permKen, permP1 = permP1, permP2 = permP2, permTest1 = permTest1, permTest2 = permTest2,
-                p.valueKen = ifelse(kendallOnly, (sum(abs(obsKen) < abs(permKen)) + 1)/(length(permKen) + 1), NA),
-                p.valueMinp1 = ifelse(minp1Only, (sum(obsP1 > permP1) + 1) / (length(permKen) + 1), NA),
-                p.valueMinp2 = ifelse(minp2Only, (sum(obsP2 > permP2) + 1) / (length(permKen) + 1), NA),
-                perm.data.Ken = perm.data.Ken, perm.data.P1 = perm.data.P1, perm.data.P2 = perm.data.P2,
-                kendallOnly = kendallOnly, minp1Only = minp1Only, minp2Only = minp2Only,
-                sampling = sampling, permSize = permSize, random.seed = seed)
-    class(out) <- "permDep"
-    return(out)
-}
-
-## gpdPv <- function(permT, obsT) {
-##     permT <- permT[order(permT, decreasing = TRUE)]
-##     ## default starting value is 250
-##     for (i in 0:24) {
-##         tmp <- permT[1:(250 - 10 * i)]
-##         ## Use bootstrap gof test, this depends on package goft
-##         ## or we can use ks.test but this require to estimate the mle first
-##         gofp <- gpd.test(tmp, 1000)$boot.test$p.value
-##         if (gofp > 0.05) {
-##             init <- c(sqrt(6 * var(tmp)) / pi, 0.1)
-##             x <- optim(init, gpdlik, dat = tmp - mean(permT[250 - 10 * i], permT[251 - 10 * i]),
-##                        hessian = TRUE, method = "Nelder-Mead")
-##             break
-##         }
-##         ## if(i == 25) {
-##         ##     tmp <- permT[1:300]
-##         ##     init <- c(sqrt(6 * var(tmp)) / pi, 0.1)
-##         ##     x <- optim(init, gpdlik, dat = tmp - mean(permT[300], permT[301]),
-##         ##                hessian = TRUE, method = "Nelder-Mead")
-##         ## }
-##     }
-##     (250 - 10 * i) * (1 - pgpd(obsT, -x$par[1], x$par[2])) / length(permT)
-## }
